#!/bin/sh  
# \
exec oagtclsh "$0" "$@"

set auto_path [linsert $auto_path 0  /usr/local/oag/apps/lib/$env(HOST_ARCH)]
set auto_path [linsert $auto_path 0 /usr/local/oag/lib_patch/$env(HOST_ARCH)]

APSStandardSetup

set RCPV APS:RunControlFixed0RC

proc APSRunControlInit {args} {
    global apsRunControlInitialized apsScriptUser apsScriptHost apsScriptDate
    global apsScriptCommand
    set pv ""
    set description ""
    set timeout 5000
    APSParseArguments {pv description timeout}

    set pvnameVAL ${pv}.VAL
    set pvnameHBT ${pv}.HBT
    set pvnameUSER ${pv}.USER
    set pvnameHOST ${pv}.HOST
    set pvnamePID ${pv}.PID
    set pvnameSEM ${pv}.SEM
    set pvnameDESC ${pv}.DESC
    set pvnameSTRT ${pv}.STRT
    set pvnameABRT ${pv}.ABRT
    set pvnameSUSP ${pv}.SUSP
    set pvnameHBTO ${pv}.HBTO
    set pvnameMSG ${pv}.MSG
    set pvnameALRM ${pv}.ALRM
    set pvnameCLR ${pv}.CLR

    # Connect up PVs
    set varList {pvVAL pvHBT pvUSER pvHOST pvPID pvSEM pvDESC pvSTRT pvABRT \
                   pvSUSP pvHBTO pvMSG pvALRM pvCLR}
    set nameList [list $pvnameVAL $pvnameHBT $pvnameUSER $pvnameHOST \
                    $pvnamePID $pvnameSEM $pvnameDESC $pvnameSTRT $pvnameABRT \
                    $pvnameSUSP $pvnameHBTO $pvnameMSG $pvnameALRM $pvnameCLR]
    if {[pv linkw $varList $nameList]} {
        return -code error "RUNCONTROL_ERROR"
    }

    eval global $varList

    # Set heartbeat timeout
    set pvHBT $timeout
    if {[pv putw pvHBT]} {
        return -code error "RUNCONTROL_ERROR"
    }

    # Try to take semaphore by putting a 0. Generates error if unavailable.
    set pvSEM 0
    if {[pv putw pvSEM 5]} {
        return -code error "RUNCONTROL_DENIED"
    }

    # Load record with user, host, pid, start time, and description
    set pvUSER $apsScriptUser
    set pvHOST $apsScriptHost
    set pvSTRT $apsScriptDate
    set pvDESC $description
    set pvPID [pid]
    if {[pv putw {pvUSER pvHOST pvSTRT pvDESC pvPID}]} {
        return -code error "RUNCONTROL_ERROR"
    }
    set apsRunControlInitialized 1
    return -code ok "RUNCONTROL_OK"
}

proc APSRunControlPing {} {
    global apsRunControlInitialized

    if {!$apsRunControlInitialized} {
        return -code error "RUNCONTROL_ERROR"
    }

    set varList {pvVAL pvABRT pvSUSP pvHBTO}

    eval global $varList

    if {[pv getw $varList]} {
        return -code error "RUNCONTROL_ERROR"
    }
    if {![string compare $pvABRT Abort]} {
        return -code error "RUNCONTROL_ABORT"
    } elseif {![string compare $pvHBTO Timeout]} {
        return -code error "RUNCONTROL_TIMEOUT"
    } elseif {![string compare $pvSUSP Suspend]} {
        while {![string compare $pvSUSP Suspend]} {
            after 1000
            update
            if {[pv getw {pvABRT pvSUSP}]} {
                return -code error "RUNCONTROL_ERROR"
            }
            if {![string compare $pvABRT Abort]} {
                return -code error "RUNCONTROL_ABORT"
            }
        }
    }
    
    # No suspend, abort, or timeout. So ping the record
    set pvVAL 1
    if {[pv putw pvVAL]} {
        return -code error "RUNCONTROL_ERROR"
    }
    return -code ok "RUNCONTROL_OK"
}

proc APSRunControlLogMessage {args} {
    global apsRunControlInitialized pvMSG pvALRM

    if {!$apsRunControlInitialized} {
        return -code error "RUNCONTROL_ERROR"
    }
    set message ""
    set severity 0
    APSParseArguments {message severity}

    set pvMSG $message
    set pvALRM $severity

    if {[pv putw {pvMSG pvALRM}]} {
        return -code error "RUNCONTROL_ERROR"
    }
    return -code ok "RUNCONTROL_OK"
}

proc APSRunControlExit {} {
    global apsRunControlInitialized

    if {!$apsRunControlInitialized} {
        return -code error "RUNCONTROL_ERROR"
    }
    set varList {pvVAL pvHBT pvUSER pvHOST pvPID pvSEM pvDESC pvSTRT pvABRT \
                   pvSUSP pvHBTO pvMSG pvALRM}
    eval global $varList
    set pvSEM 1
    if {[pv putw pvSEM]} {
        return -code error "RUNCONTROL_ERROR"
    }
    if {[pv unlink $varList]} {
        return -code error "RUNCONTROL_ERROR"
    }
    set apsRunControlInitialized 0
    return -code ok "RUNCONTROL_OK"
}


#Initialize Run Control record with a 10 second timeout
if {[catch {APSRunControlInit -pv $RCPV \
              -description "TEST RunControl" \
              -timeout 10000} result]} {
    puts stderr "Error: $result"
    exit
}
#Set Run Control message
if {[catch {APSRunControlLogMessage -message "Running"} result]} {
    puts stderr "Error: $result"
    exit
}
#Ping Run Control
if {[catch {APSRunControlPing} result]} {
    puts stderr "Ending via ping from runControl10:$result"
    exit 1
}
#Do some work
after 5000
#Ping Run Control
if {[catch {APSRunControlPing} result]} {
    puts stderr "Ending via ping from runControl10:$result"
    exit 1
}
#Release Run Control
catch {APSRunControlExit}
exit

